#include "widget.h"
#include "ui_widget.h"
#include <QFile>
#include <QDebug>
#include <QPainter>
#include <QString>
#include <string.h>
#include <QThread>
#include <iostream>
using namespace  std;

#include "qmqtt.h"

#define TITLE "client MQTT - 1.00\t\t"

// __________________________ constructeur
Widget::Widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Widget)
{
    ui->setupUi(this);


    // default values
    brokerHost = "localhost";

    // topics to subscribe
    topicsSubscribed<<"$sys/#";
    topicsSubscribed<<"sensors/#";
    topicsSubscribed<<"actuators/#";


    // MQTT cleint
    client = new QMQTT::Client(brokerHost);
    connect(client, SIGNAL(received(const QMQTT::Message&)), this, SLOT(messageReceived(const QMQTT::Message&)));
    connect(client, SIGNAL(connacked(quint8)), this, SLOT(connackedReceived(quint8)));
    connect(client, SIGNAL(published(QMQTT::Message&)), this, SLOT(pubackedReceived(QMQTT::Message&)));
    msgId = 1;          // message number
    qos = 0;            // quality of service
    retain = false;     // no retain on broker

    // le titre de la fenetre
    this->setWindowTitle(TITLE);
}

//_____________________________________________Destructeur
Widget::~Widget()
{
    delete ui;
    if (client) delete client;
}

//      slot : MQTT message Received
void Widget::messageReceived(const QMQTT::Message &message){
    // écriture sur l'interface tout simplement
    ui->textEditMessage->append(message.topic()+"="+message.payload());

}

//      slot : Connect acknoledge Received
void Widget::connackedReceived(quint8 code){
    // modifie le titre de la fenêtre
    this->setWindowTitle(TITLE+brokerHost);
    // affiche les topics souscrits
    ui->textEditSubscribed->clear();
    for (int i=0 ; i<topicsSubscribed.size() ; i++){
            client->subscribe(topicsSubscribed[i], qos);                // souscription
            ui->textEditSubscribed->append(topicsSubscribed[i]);        // affichage
        }
}

//      slot :  publish Acknoledge
void Widget::pubackedReceived(QMQTT::Message& message){
}

//      bouton connecter/déconnecter
void Widget::on_toolButtonConnect_clicked()
{
    if (client->isConnected()){
                                client->disconnect();
                                this->setWindowTitle(TITLE);
                                ui->toolButtonConnect->setText("connect");
                                }
                        else     {
                                client->setHost((brokerHost = ui->lineEdithost->text()));
                                client->connect();
                                ui->toolButtonConnect->setText("disconnect");
                                ui->textEditMessage->clear();       // effacement de la fenetre des messages
                                }
}

//  boution publish
void Widget::on_pushButtonPublish_clicked()
{

QMQTT::Message message;         // un message MQTT
// récupération des données sur l'interface
message.setQos(ui->comboBoxQOS->currentIndex());
message.setRetain(ui->checkBoxRetain->isChecked());
message.setTopic(ui->lineEditTopic->text());
message.setPayload(ui->lineEditMessage->text().toLatin1());

client->publish(message);
}
